#version 430

layout (binding = 1) uniform sampler2D worldPositionSamp;
layout (binding = 2) uniform sampler2D worldNormalSamp;
layout (binding = 3) uniform sampler2D worldColorSamp;
layout (binding = 4) uniform sampler2D worldDepthSamp;
layout (binding = 5) uniform sampler2D noiseSamp;

const float bias = 0.025;
const float radius = 0.5;
const int kernelSize = 64;
uniform vec3 ssaoKernel[kernelSize];
uniform mat4 projMatrix;

in vec2 varTextureCoord;
out vec4 color;

uniform vec3 viewPos;

const vec2 noiseScale = vec2(1024.0 / 4.0, 620.0 / 4.0);

void main()
{     
    vec3 fragPos = texture(worldPositionSamp, varTextureCoord).xyz;
    vec3 normal = texture(worldNormalSamp, varTextureCoord).rgb;
    vec3 randomVec = texture(noiseSamp, varTextureCoord * noiseScale).xyz; 

    vec3 tangent = normalize(randomVec - normal * dot(randomVec, normal));
    vec3 bitangent = cross(normal, tangent);
    mat3 TBN = mat3(tangent, bitangent, normal);  

    float occlusion = 0.0;
    for(int i = 0; i < kernelSize; i++)
    {
        // get sample position
        vec3 sampl = TBN * ssaoKernel[i]; // From tangent to view-space
        sampl = fragPos + sampl * radius; 

        vec4 offset = vec4(sampl, 1.0);
        offset = projMatrix * offset;    // from view to clip-space
        offset.xyz /= offset.w;               // perspective divide
        offset.xyz = offset.xyz * 0.5 + 0.5; // transform to range 0.0 - 1.0  

        float sampleDepth = texture(worldPositionSamp, offset.xy).z; 

        //float rangeCheck = smoothstep(0.0, 1.0, radius / abs(fragPos.z - sampleDepth));
        //occlusion += float(sampleDepth >= sampl.z + bias) * rangeCheck;   
        occlusion += float(sampleDepth >= sampl.z + bias);  
    }  
    
    occlusion /= kernelSize;
    occlusion = 1 - occlusion;
    color = vec4(occlusion, occlusion, occlusion, 1);
} 